package com.sun.media.sound;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.BooleanControl;
import javax.sound.sampled.FloatControl;
import javax.sound.sampled.Control;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.TargetDataLine;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.AudioFormat;
import java.util.Vector;
import javax.sound.sampled.Clip;
import javax.sound.sampled.Line;

final class DirectAudioDevice extends AbstractMixer
{
    private static final int CLIP_BUFFER_TIME = 1000;
    private static final int DEFAULT_LINE_BUFFER_TIME = 500;
    private int deviceCountOpened;
    private int deviceCountStarted;
    
    DirectAudioDevice(DirectAudioDeviceProvider.DirectAudioDeviceInfo directAudioDeviceInfo) {
        super(directAudioDeviceInfo, null, null, (Line$Info[])null);
        this.deviceCountOpened = 0;
        this.deviceCountStarted = 0;
        DirectAudioDevice.DirectDLI dataLineInfo = this.createDataLineInfo(true);
        if (dataLineInfo != null) {
            (this.sourceLineInfo = new Line.Info[2])[0] = dataLineInfo;
            this.sourceLineInfo[1] = new DirectAudioDevice.DirectDLI(Clip.class, dataLineInfo.getFormats(), DirectAudioDevice.DirectDLI.access$000(dataLineInfo), 32, -1);
        }
        else {
            this.sourceLineInfo = new Line.Info[0];
        }
        DirectAudioDevice.DirectDLI dataLineInfo2 = this.createDataLineInfo(false);
        if (dataLineInfo2 != null) {
            (this.targetLineInfo = new Line.Info[1])[0] = dataLineInfo2;
        }
        else {
            this.targetLineInfo = new Line.Info[0];
        }
    }
    
    private DirectAudioDevice.DirectDLI createDataLineInfo(boolean b) {
        Vector<Object> vector = new Vector();
        AudioFormat[] array = null;
        AudioFormat[] array2 = null;
        synchronized (vector) {
            nGetFormats(this.getMixerIndex(), this.getDeviceID(), b, vector);
            if (vector.size() > 0) {
                int size = vector.size();
                array = new AudioFormat[size];
                for (int i = 0; i < size; ++i) {
                    AudioFormat audioFormat = (AudioFormat)vector.elementAt(i);
                    (array[i] = audioFormat).getSampleSizeInBits();
                    int equals = audioFormat.getEncoding().equals(AudioFormat.Encoding.PCM_SIGNED);
                    int equals2 = audioFormat.getEncoding().equals(AudioFormat.Encoding.PCM_UNSIGNED);
                    if (equals || equals2) {
                        ++size;
                    }
                }
                array2 = new AudioFormat[size];
                int n = 0;
                for (AudioFormat audioFormat2 : array) {
                    array2[n++] = audioFormat2;
                    int sampleSizeInBits = audioFormat2.getSampleSizeInBits();
                    int equals3 = audioFormat2.getEncoding().equals(AudioFormat.Encoding.PCM_SIGNED);
                    int equals4 = audioFormat2.getEncoding().equals(AudioFormat.Encoding.PCM_UNSIGNED);
                    if (sampleSizeInBits == 8) {
                        if (!(equals3 == 0)) {
                            array2[n++] = new AudioFormat(AudioFormat.Encoding.PCM_UNSIGNED, audioFormat2.getSampleRate(), sampleSizeInBits, audioFormat2.getChannels(), audioFormat2.getFrameSize(), audioFormat2.getSampleRate(), audioFormat2.isBigEndian());
                        }
                        else if (!(equals4 == 0)) {
                            array2[n++] = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, audioFormat2.getSampleRate(), sampleSizeInBits, audioFormat2.getChannels(), audioFormat2.getFrameSize(), audioFormat2.getSampleRate(), audioFormat2.isBigEndian());
                        }
                    }
                    else if (sampleSizeInBits > 8 && (equals3 || equals4)) {
                        array2[n++] = new AudioFormat(audioFormat2.getEncoding(), audioFormat2.getSampleRate(), sampleSizeInBits, audioFormat2.getChannels(), audioFormat2.getFrameSize(), audioFormat2.getSampleRate(), !audioFormat2.isBigEndian());
                    }
                }
            }
        }
        if (array2 != null) {
            return new DirectAudioDevice.DirectDLI(b ? SourceDataLine.class : TargetDataLine.class, array2, array, 32, -1);
        }
        return null;
    }
    
    @Override
    public Line getLine(Line.Info info) throws LineUnavailableException {
        Line.Info lineInfo = this.getLineInfo(info);
        if (lineInfo == null) {
            throw new IllegalArgumentException("Line unsupported: " + info);
        }
        if ((lineInfo instanceof DataLine.Info)) {
            DataLine.Info info2 = (DataLine.Info)lineInfo;
            int maxBufferSize = -1;
            AudioFormat[] formats = null;
            if ((info instanceof DataLine.Info)) {
                formats = ((DataLine.Info)info).getFormats();
                maxBufferSize = ((DataLine.Info)info).getMaxBufferSize();
            }
            AudioFormat audioFormat;
            if (formats == null || formats.length == 0) {
                audioFormat = null;
            }
            else {
                audioFormat = formats[formats.length - 1];
                if (!Toolkit.isFullySpecifiedPCMFormat(audioFormat)) {
                    audioFormat = null;
                }
            }
            if (info2.getLineClass().isAssignableFrom(DirectAudioDevice.DirectSDL.class)) {
                return new DirectAudioDevice.DirectSDL(info2, audioFormat, maxBufferSize, this);
            }
            if (info2.getLineClass().isAssignableFrom(DirectAudioDevice.DirectClip.class)) {
                return new DirectAudioDevice.DirectClip(info2, audioFormat, maxBufferSize, this);
            }
            if (info2.getLineClass().isAssignableFrom(DirectAudioDevice.DirectTDL.class)) {
                return new DirectAudioDevice.DirectTDL(info2, audioFormat, maxBufferSize, this);
            }
        }
        throw new IllegalArgumentException("Line unsupported: " + info);
    }
    
    @Override
    public int getMaxLines(Line.Info info) {
        Line.Info lineInfo = this.getLineInfo(info);
        if (lineInfo == null) {
            return 0;
        }
        if ((lineInfo instanceof DataLine.Info)) {
            return this.getMaxSimulLines();
        }
        return 0;
    }
    
    @Override
    protected void implOpen() throws LineUnavailableException {
    }
    
    @Override
    protected void implClose() {
    }
    
    @Override
    protected void implStart() {
    }
    
    @Override
    protected void implStop() {
    }
    
    int getMixerIndex() {
        return ((DirectAudioDeviceProvider.DirectAudioDeviceInfo)this.getMixerInfo()).getIndex();
    }
    
    int getDeviceID() {
        return ((DirectAudioDeviceProvider.DirectAudioDeviceInfo)this.getMixerInfo()).getDeviceID();
    }
    
    int getMaxSimulLines() {
        return ((DirectAudioDeviceProvider.DirectAudioDeviceInfo)this.getMixerInfo()).getMaxSimulLines();
    }
    
    private static void addFormat(Vector vector, int sampleSizeInBits, int frameSize, int channels, float n, int n2, boolean b, boolean bigEndian) {
        AudioFormat.Encoding encoding = null;
        switch (n2) {
            case 0: {
                encoding = (b ? AudioFormat.Encoding.PCM_SIGNED : AudioFormat.Encoding.PCM_UNSIGNED);
                break;
            }
            case 1: {
                encoding = AudioFormat.Encoding.ULAW;
                if (sampleSizeInBits != 8) {
                    sampleSizeInBits = 8;
                    frameSize = channels;
                    break;
                }
                break;
            }
            case 2: {
                encoding = AudioFormat.Encoding.ALAW;
                if (sampleSizeInBits != 8) {
                    sampleSizeInBits = 8;
                    frameSize = channels;
                    break;
                }
                break;
            }
        }
        if (encoding == null) {
            return;
        }
        if (frameSize <= 0) {
            if (channels > 0) {
                frameSize = (sampleSizeInBits + 7) / 8 * channels;
            }
            else {
                frameSize = -1;
            }
        }
        vector.add(new AudioFormat(encoding, n, sampleSizeInBits, channels, frameSize, n, bigEndian));
        return;
    }
    
    protected static AudioFormat getSignOrEndianChangedFormat(AudioFormat audioFormat) {
        int equals = audioFormat.getEncoding().equals(AudioFormat.Encoding.PCM_SIGNED);
        int equals2 = audioFormat.getEncoding().equals(AudioFormat.Encoding.PCM_UNSIGNED);
        if (audioFormat.getSampleSizeInBits() > 8 && equals) {
            return new AudioFormat(audioFormat.getEncoding(), audioFormat.getSampleRate(), audioFormat.getSampleSizeInBits(), audioFormat.getChannels(), audioFormat.getFrameSize(), audioFormat.getFrameRate(), !audioFormat.isBigEndian());
        }
        if (audioFormat.getSampleSizeInBits() == 8 && (equals || equals2)) {
            return new AudioFormat(equals ? AudioFormat.Encoding.PCM_UNSIGNED : AudioFormat.Encoding.PCM_SIGNED, audioFormat.getSampleRate(), audioFormat.getSampleSizeInBits(), audioFormat.getChannels(), audioFormat.getFrameSize(), audioFormat.getFrameRate(), audioFormat.isBigEndian());
        }
        return null;
    }
    
    private static native void nGetFormats(int p0, int p1, boolean p2, Vector p3);
    
    private static native long nOpen(int p0, int p1, boolean p2, int p3, float p4, int p5, int p6, int p7, boolean p8, boolean p9, int p10) throws LineUnavailableException;
    
    private static native void nStart(long p0, boolean p1);
    
    private static native void nStop(long p0, boolean p1);
    
    private static native void nClose(long p0, boolean p1);
    
    private static native int nWrite(long p0, byte[] p1, int p2, int p3, int p4, float p5, float p6);
    
    private static native int nRead(long p0, byte[] p1, int p2, int p3, int p4);
    
    private static native int nGetBufferSize(long p0, boolean p1);
    
    private static native boolean nIsStillDraining(long p0, boolean p1);
    
    private static native void nFlush(long p0, boolean p1);
    
    private static native int nAvailable(long p0, boolean p1);
    
    private static native long nGetBytePosition(long p0, boolean p1, long p2);
    
    private static native void nSetBytePosition(long p0, boolean p1, long p2);
    
    private static native boolean nRequiresServicing(long p0, boolean p1);
    
    private static native void nService(long p0, boolean p1);
    
    static long access$900(int n, int n2, boolean b, int n3, float n4, int n5, int n6, int n7, boolean b2, boolean b3, int n8) throws LineUnavailableException {
        return nOpen(n, n2, b, n3, n4, n5, n6, n7, b2, b3, n8);
    }
    
    static int access$1000(long n, boolean b) {
        return nGetBufferSize(n, b);
    }
    
    static void access$1100(long n, boolean b) {
        nStart(n, b);
    }
    
    static void access$1200(long n, boolean b) {
        nStop(n, b);
    }
    
    static void access$1300(long n, boolean b) {
        nClose(n, b);
    }
    
    static int access$1400(long n, boolean b) {
        return nAvailable(n, b);
    }
    
    static boolean access$1500(long n, boolean b) {
        return nIsStillDraining(n, b);
    }
    
    static void access$1600(long n, boolean b) {
        nFlush(n, b);
    }
    
    static long access$1700(long n, boolean b, long n2) {
        return nGetBytePosition(n, b, n2);
    }
    
    static int access$1800(long n, byte[] array, int n2, int n3, int n4, float n5, float n6) {
        return nWrite(n, array, n2, n3, n4, n5, n6);
    }
    
    static boolean access$1900(long n, boolean b) {
        return nRequiresServicing(n, b);
    }
    
    static void access$2000(long n, boolean b) {
        nService(n, b);
    }
    
    static int access$2400(long n, byte[] array, int n2, int n3, int n4) {
        return nRead(n, array, n2, n3, n4);
    }
    
    static void access$2500(long n, boolean b, long n2) {
        nSetBytePosition(n, b, n2);
    }
    
    private static final class DirectDLI extends DataLine.Info
    {
        final AudioFormat[] hardwareFormats;
        
        private DirectDLI(Class lineClass, AudioFormat[] formats, AudioFormat[] hardwareFormats, int minBufferSize, int maxBufferSize) {
            super(lineClass, formats, minBufferSize, maxBufferSize);
            this.hardwareFormats = hardwareFormats;
        }
        
        public boolean isFormatSupportedInHardware(AudioFormat audioFormat) {
            if (audioFormat == null) {
                return false;
            }
            int i = 0;
            while (i < this.hardwareFormats.length) {
                if (audioFormat.matches(this.hardwareFormats[i])) {
                    return true;
                }
                ++i;
            }
            return false;
        }
        
        private AudioFormat[] getHardwareFormats() {
            return this.hardwareFormats;
        }
        
        static AudioFormat[] access$000(DirectAudioDevice.DirectDLI directDLI) {
            return directDLI.getHardwareFormats();
        }
    }
    
    private static class DirectDL extends AbstractDataLine implements EventDispatcher.LineMonitor
    {
        protected final int mixerIndex;
        protected final int deviceID;
        protected long id;
        protected int waitTime;
        protected volatile boolean flushing;
        protected final boolean isSource;
        protected volatile long bytePosition;
        protected volatile boolean doIO;
        protected volatile boolean stoppedWritten;
        protected volatile boolean drained;
        protected boolean monitoring;
        protected int softwareConversionSize;
        protected AudioFormat hardwareFormat;
        private final DirectAudioDevice.DirectDL.Gain gainControl;
        private final DirectAudioDevice.DirectDL.Mute muteControl;
        private final DirectAudioDevice.DirectDL.Balance balanceControl;
        private final DirectAudioDevice.DirectDL.Pan panControl;
        private float leftGain;
        private float rightGain;
        protected volatile boolean noService;
        protected final Object lockNative;
        
        protected DirectDL(DataLine.Info info, DirectAudioDevice directAudioDevice, AudioFormat audioFormat, int n, int mixerIndex, int deviceID, boolean isSource) {
            super(info, directAudioDevice, null, audioFormat, n);
            this.flushing = false;
            this.doIO = false;
            this.stoppedWritten = false;
            this.drained = false;
            this.monitoring = false;
            this.softwareConversionSize = 0;
            this.gainControl = new DirectAudioDevice.DirectDL.Gain();
            this.muteControl = new DirectAudioDevice.DirectDL.Mute();
            this.balanceControl = new DirectAudioDevice.DirectDL.Balance();
            this.panControl = new DirectAudioDevice.DirectDL.Pan();
            this.noService = false;
            this.lockNative = new Object();
            this.mixerIndex = mixerIndex;
            this.deviceID = deviceID;
            this.waitTime = 10;
            this.isSource = isSource;
        }
        
        @Override
        void implOpen(AudioFormat obj, int bufferSize) throws LineUnavailableException {
            Toolkit.isFullySpecifiedAudioFormat(obj);
            if (!this.isSource) {
                JSSecurityManager.checkRecordPermission();
            }
            int n = 0;
            if (obj.getEncoding().equals(AudioFormat.Encoding.ULAW)) {
                n = 1;
            }
            else if (obj.getEncoding().equals(AudioFormat.Encoding.ALAW)) {
                n = 2;
            }
            if (bufferSize <= -1) {
                bufferSize = (int)Toolkit.millis2bytes(obj, 500L);
            }
            DirectAudioDevice.DirectDLI directDLI = null;
            if ((this.info instanceof DirectAudioDevice.DirectDLI)) {
                directDLI = (DirectAudioDevice.DirectDLI)this.info;
            }
            if (this.isSource) {
                if (!obj.getEncoding().equals(AudioFormat.Encoding.PCM_SIGNED) && !obj.getEncoding().equals(AudioFormat.Encoding.PCM_UNSIGNED)) {
                    this.controls = new Control[0];
                }
                else if (obj.getChannels() > 2 || obj.getSampleSizeInBits() > 16) {
                    this.controls = new Control[0];
                }
                else {
                    if (obj.getChannels() == 1) {
                        this.controls = new Control[2];
                    }
                    else {
                        (this.controls = new Control[4])[2] = this.balanceControl;
                        this.controls[3] = this.panControl;
                    }
                    this.controls[0] = this.gainControl;
                    this.controls[1] = this.muteControl;
                }
            }
            this.hardwareFormat = obj;
            this.softwareConversionSize = 0;
            if (directDLI != null && !directDLI.isFormatSupportedInHardware(obj)) {
                AudioFormat signOrEndianChangedFormat = DirectAudioDevice.getSignOrEndianChangedFormat(obj);
                if (directDLI.isFormatSupportedInHardware(signOrEndianChangedFormat)) {
                    this.hardwareFormat = signOrEndianChangedFormat;
                    this.softwareConversionSize = obj.getFrameSize() / obj.getChannels();
                }
            }
            bufferSize = bufferSize / obj.getFrameSize() * obj.getFrameSize();
            this.id = DirectAudioDevice.access$900(this.mixerIndex, this.deviceID, this.isSource, n, this.hardwareFormat.getSampleRate(), this.hardwareFormat.getSampleSizeInBits(), this.hardwareFormat.getFrameSize(), this.hardwareFormat.getChannels(), this.hardwareFormat.getEncoding().equals(AudioFormat.Encoding.PCM_SIGNED), this.hardwareFormat.isBigEndian(), bufferSize);
            if (this.id == 0L) {
                throw new LineUnavailableException("line with format " + obj + " not supported.");
            }
            this.bufferSize = DirectAudioDevice.access$1000(this.id, this.isSource);
            if (this.bufferSize < 1) {
                this.bufferSize = bufferSize;
            }
            this.format = obj;
            this.waitTime = (int)Toolkit.bytes2millis(obj, (long)this.bufferSize) / 4;
            if (this.waitTime < 10) {
                this.waitTime = 1;
            }
            else if (this.waitTime > 1000) {
                this.waitTime = 1000;
            }
            this.bytePosition = 0L;
            this.stoppedWritten = false;
            this.doIO = false;
            this.calcVolume();
            return;
        }
        
        @Override
        void implStart() {
            if (!this.isSource) {
                JSSecurityManager.checkRecordPermission();
            }
            synchronized (this.lockNative) {
                DirectAudioDevice.access$1100(this.id, this.isSource);
            }
            this.monitoring = this.requiresServicing();
            if (this.monitoring) {
                this.getEventDispatcher().addLineMonitor(this);
            }
            synchronized (this.lock) {
                this.doIO = true;
                if (this.isSource && this.stoppedWritten) {
                    this.setStarted(true);
                    this.setActive(true);
                }
            }
        }
        
        @Override
        void implStop() {
            if (!this.isSource) {
                JSSecurityManager.checkRecordPermission();
            }
            if (this.monitoring) {
                this.getEventDispatcher().removeLineMonitor(this);
                this.monitoring = false;
            }
            synchronized (this.lockNative) {
                DirectAudioDevice.access$1200(this.id, this.isSource);
            }
            synchronized (this.lock) {
                this.setActive(this.doIO = false);
                this.setStarted(false);
                this.lock.notifyAll();
            }
            this.stoppedWritten = false;
        }
        
        @Override
        void implClose() {
            if (!this.isSource) {
                JSSecurityManager.checkRecordPermission();
            }
            if (this.monitoring) {
                this.getEventDispatcher().removeLineMonitor(this);
                this.monitoring = false;
            }
            this.doIO = false;
            long id = this.id;
            this.id = 0L;
            synchronized (this.lockNative) {
                DirectAudioDevice.access$1300(id, this.isSource);
            }
            this.bytePosition = 0L;
            this.softwareConversionSize = 0;
        }
        
        @Override
        public int available() {
            if (this.id == 0L) {
                return 0;
            }
            int access$1400;
            synchronized (this.lockNative) {
                access$1400 = DirectAudioDevice.access$1400(this.id, this.isSource);
            }
            return access$1400;
        }
        
        @Override
        public void drain() {
            this.noService = true;
            int n = 0;
            long longFramePosition = this.getLongFramePosition();
            boolean n2 = false;
            while (!this.drained) {
                synchronized (this.lockNative) {
                    if (this.id == 0L || !this.doIO || !DirectAudioDevice.access$1500(this.id, this.isSource)) {
                        break;
                    }
                }
                if (n % 5 == 4) {
                    long longFramePosition2 = this.getLongFramePosition();
                    n2 |= (longFramePosition2 != longFramePosition);
                    if (n % 50 > 45) {
                        if (n2 == 0) {
                            break;
                        }
                        n2 = false;
                        longFramePosition = longFramePosition2;
                    }
                }
                ++n;
                synchronized (this.lock) {
                    try {
                        this.lock.wait(10L);
                    }
                    catch (InterruptedException ex) {}
                }
            }
            if (this.doIO && this.id != 0L) {
                this.drained = true;
            }
            this.noService = false;
        }
        
        @Override
        public void flush() {
            if (this.id != 0L) {
                this.flushing = true;
                synchronized (this.lock) {
                    this.lock.notifyAll();
                }
                synchronized (this.lockNative) {
                    if (this.id != 0L) {
                        DirectAudioDevice.access$1600(this.id, this.isSource);
                    }
                }
                this.drained = true;
            }
        }
        
        @Override
        public long getLongFramePosition() {
            long access$1700;
            synchronized (this.lockNative) {
                access$1700 = DirectAudioDevice.access$1700(this.id, this.isSource, this.bytePosition);
            }
            if (access$1700 < 0L) {
                access$1700 = 0L;
            }
            return access$1700 / (long)this.getFormat().getFrameSize();
        }
        
        public int write(byte[] array, int index, int n) {
            this.flushing = false;
            if (n == 0) {
                return 0;
            }
            if (n < 0) {
                throw new IllegalArgumentException("illegal len: " + n);
            }
            if (n % this.getFormat().getFrameSize() != 0) {
                throw new IllegalArgumentException("illegal request to write non-integral number of frames (" + n + " bytes, frameSize = " + this.getFormat().getFrameSize() + " bytes)");
            }
            if (index < 0) {
                throw new ArrayIndexOutOfBoundsException(index);
            }
            if ((long)index + (long)n > (long)array.length) {
                throw new ArrayIndexOutOfBoundsException(array.length);
            }
            synchronized (this.lock) {
                if (!this.isActive() && this.doIO) {
                    this.setActive(true);
                    this.setStarted(true);
                }
            }
            int n2 = 0;
            while (!this.flushing) {
                int access$1800;
                synchronized (this.lockNative) {
                    access$1800 = DirectAudioDevice.access$1800(this.id, array, index, n, this.softwareConversionSize, this.leftGain, this.rightGain);
                    if (access$1800 < 0) {
                        break;
                    }
                    this.bytePosition += (long)access$1800;
                    if (access$1800 > 0) {
                        this.drained = false;
                    }
                }
                n -= access$1800;
                n2 += access$1800;
                if (this.doIO && n > 0) {
                    index += access$1800;
                    synchronized (this.lock) {
                        try {
                            this.lock.wait((long)this.waitTime);
                        }
                        catch (InterruptedException ex) {}
                    }
                }
                else {
                    break;
                }
            }
            if (n2 > 0 && !this.doIO) {
                this.stoppedWritten = true;
            }
            return n2;
        }
        
        protected boolean requiresServicing() {
            return DirectAudioDevice.access$1900(this.id, this.isSource);
        }
        
        @Override
        public void checkLine() {
            synchronized (this.lockNative) {
                if (this.monitoring && this.doIO && this.id != 0L && !this.flushing && !this.noService) {
                    DirectAudioDevice.access$2000(this.id, this.isSource);
                }
            }
        }
        
        private void calcVolume() {
            if (this.getFormat() == null) {
                return;
            }
            if (this.muteControl.getValue()) {
                this.leftGain = 0.0f;
                this.rightGain = 0.0f;
                return;
            }
            float linearGain = this.gainControl.getLinearGain();
            if (this.getFormat().getChannels() == 1) {
                this.leftGain = linearGain;
                this.rightGain = linearGain;
            }
            else {
                float value = this.balanceControl.getValue();
                if (value < 0.0f) {
                    this.leftGain = linearGain;
                    this.rightGain = linearGain * (value + 1.0f);
                }
                else {
                    this.leftGain = linearGain * (1.0f - value);
                    this.rightGain = linearGain;
                }
            }
            return;
        }
        
        static void access$2100(DirectAudioDevice.DirectDL directDL) {
            directDL.calcVolume();
        }
        
        protected final class Gain extends FloatControl
        {
            private float linearGain;
            
            private Gain() {
                super(FloatControl.Type.MASTER_GAIN, Toolkit.linearToDB(0.0f), Toolkit.linearToDB(2.0f), Math.abs(Toolkit.linearToDB(1.0f) - Toolkit.linearToDB(0.0f)) / 128.0f, -1, 0.0f, "dB", "Minimum", "", "Maximum");
                this.linearGain = 1.0f;
            }
            
            @Override
            public void setValue(float n) {
                float dbToLinear = Toolkit.dBToLinear(n);
                super.setValue(Toolkit.linearToDB(dbToLinear));
                this.linearGain = dbToLinear;
                DirectAudioDevice.DirectDL.access$2100(DirectAudioDevice.DirectDL.this);
            }
            
            float getLinearGain() {
                return this.linearGain;
            }
        }
        
        private final class Mute extends BooleanControl
        {
            private Mute() {
                super(BooleanControl.Type.MUTE, false, "True", "False");
            }
            
            @Override
            public void setValue(boolean value) {
                super.setValue(value);
                DirectAudioDevice.DirectDL.access$2100(DirectAudioDevice.DirectDL.this);
            }
        }
        
        private final class Balance extends FloatControl
        {
            private Balance() {
                super(FloatControl.Type.BALANCE, -1.0f, 1.0f, 0.0078125f, -1, 0.0f, "", "Left", "Center", "Right");
            }
            
            @Override
            public void setValue(float n) {
                this.setValueImpl(n);
                DirectAudioDevice.DirectDL.this.panControl.setValueImpl(n);
                DirectAudioDevice.DirectDL.access$2100(DirectAudioDevice.DirectDL.this);
            }
            
            void setValueImpl(float value) {
                super.setValue(value);
            }
        }
        
        private final class Pan extends FloatControl
        {
            private Pan() {
                super(FloatControl.Type.PAN, -1.0f, 1.0f, 0.0078125f, -1, 0.0f, "", "Left", "Center", "Right");
            }
            
            @Override
            public void setValue(float n) {
                this.setValueImpl(n);
                DirectAudioDevice.DirectDL.this.balanceControl.setValueImpl(n);
                DirectAudioDevice.DirectDL.access$2100(DirectAudioDevice.DirectDL.this);
            }
            
            void setValueImpl(float value) {
                super.setValue(value);
            }
        }
    }
    
    private static final class DirectSDL extends DirectAudioDevice.DirectDL implements SourceDataLine
    {
        private DirectSDL(DataLine.Info info, AudioFormat audioFormat, int n, DirectAudioDevice directAudioDevice) {
            super(info, directAudioDevice, audioFormat, n, directAudioDevice.getMixerIndex(), directAudioDevice.getDeviceID(), true);
        }
    }
    
    private static final class DirectTDL extends DirectAudioDevice.DirectDL implements TargetDataLine
    {
        private DirectTDL(DataLine.Info info, AudioFormat audioFormat, int n, DirectAudioDevice directAudioDevice) {
            super(info, directAudioDevice, audioFormat, n, directAudioDevice.getMixerIndex(), directAudioDevice.getDeviceID(), false);
        }
        
        @Override
        public int read(byte[] array, int index, int n) {
            this.flushing = false;
            if (n == 0) {
                return 0;
            }
            if (n < 0) {
                throw new IllegalArgumentException("illegal len: " + n);
            }
            if (n % this.getFormat().getFrameSize() != 0) {
                throw new IllegalArgumentException("illegal request to read non-integral number of frames (" + n + " bytes, frameSize = " + this.getFormat().getFrameSize() + " bytes)");
            }
            if (index < 0) {
                throw new ArrayIndexOutOfBoundsException(index);
            }
            if ((long)index + (long)n > (long)array.length) {
                throw new ArrayIndexOutOfBoundsException(array.length);
            }
            synchronized (this.lock) {
                if (!this.isActive() && this.doIO) {
                    this.setActive(true);
                    this.setStarted(true);
                }
            }
            int n2 = 0;
            while (this.doIO && !this.flushing) {
                int access$2400;
                synchronized (this.lockNative) {
                    access$2400 = DirectAudioDevice.access$2400(this.id, array, index, n, this.softwareConversionSize);
                    if (access$2400 < 0) {
                        break;
                    }
                    this.bytePosition += (long)access$2400;
                    if (access$2400 > 0) {
                        this.drained = false;
                    }
                }
                n -= access$2400;
                n2 += access$2400;
                if (n > 0) {
                    index += access$2400;
                    synchronized (this.lock) {
                        try {
                            this.lock.wait((long)this.waitTime);
                        }
                        catch (InterruptedException ex) {}
                    }
                }
                else {
                    break;
                }
            }
            if (this.flushing) {
                n2 = 0;
            }
            return n2;
        }
    }
    
    private static final class DirectClip extends DirectAudioDevice.DirectDL implements Clip, Runnable, AutoClosingClip
    {
        private volatile Thread thread;
        private volatile byte[] audioData;
        private volatile int frameSize;
        private volatile int m_lengthInFrames;
        private volatile int loopCount;
        private volatile int clipBytePosition;
        private volatile int newFramePosition;
        private volatile int loopStartFrame;
        private volatile int loopEndFrame;
        private boolean autoclosing;
        
        private DirectClip(DataLine.Info info, AudioFormat audioFormat, int n, DirectAudioDevice directAudioDevice) {
            super(info, directAudioDevice, audioFormat, n, directAudioDevice.getMixerIndex(), directAudioDevice.getDeviceID(), true);
            this.audioData = null;
            this.autoclosing = false;
        }
        
        @Override
        public void open(AudioFormat audioFormat, byte[] array, int n, int n2) throws LineUnavailableException {
            Toolkit.isFullySpecifiedAudioFormat(audioFormat);
            byte[] array2 = new byte[n2];
            System.arraycopy(array, n, array2, 0, n2);
            this.open(audioFormat, array2, n2 / audioFormat.getFrameSize());
        }
        
        private void open(AudioFormat audioFormat, byte[] audioData, int lengthInFrames) throws LineUnavailableException {
            Toolkit.isFullySpecifiedAudioFormat(audioFormat);
            synchronized (this.mixer) {
                if (this.isOpen()) {
                    throw new IllegalStateException("Clip is already open with format " + this.getFormat() + " and frame lengh of " + this.getFrameLength());
                }
                this.audioData = audioData;
                this.frameSize = audioFormat.getFrameSize();
                this.m_lengthInFrames = lengthInFrames;
                this.bytePosition = 0L;
                this.clipBytePosition = 0;
                this.newFramePosition = -1;
                this.loopStartFrame = 0;
                this.loopEndFrame = lengthInFrames - 1;
                this.loopCount = 0;
                try {
                    this.open(audioFormat, (int)Toolkit.millis2bytes(audioFormat, 1000L));
                }
                catch (LineUnavailableException ex) {
                    this.audioData = null;
                    throw ex;
                }
                catch (IllegalArgumentException ex2) {
                    this.audioData = null;
                    throw ex2;
                }
                (this.thread = JSSecurityManager.createThread(this, "Direct Clip", true, 6, false)).start();
            }
            if (this.isAutoClosing()) {
                this.getEventDispatcher().autoClosingClipOpened(this);
            }
        }
        
        @Override
        public void open(AudioInputStream audioInputStream) throws LineUnavailableException, IOException {
            Toolkit.isFullySpecifiedAudioFormat(audioInputStream.getFormat());
            synchronized (this.mixer) {
                if (this.isOpen()) {
                    throw new IllegalStateException("Clip is already open with format " + this.getFormat() + " and frame lengh of " + this.getFrameLength());
                }
                int n = (int)audioInputStream.getFrameLength();
                int off = 0;
                int frameSize = audioInputStream.getFormat().getFrameSize();
                byte[] internalBuffer;
                if (n != -1) {
                    int len = n * frameSize;
                    if (len < 0) {
                        throw new IllegalArgumentException("Audio data < 0");
                    }
                    try {
                        internalBuffer = new byte[len];
                    }
                    catch (OutOfMemoryError outOfMemoryError2) {
                        throw new IOException("Audio data is too big");
                    }
                    int read = 0;
                    while (len > 0 && read >= 0) {
                        read = audioInputStream.read(internalBuffer, off, len);
                        if (read > 0) {
                            off += read;
                            len -= read;
                        }
                        else {
                            if (read != 0) {
                                continue;
                            }
                            Thread.yield();
                        }
                    }
                }
                else {
                    int max = Math.max(16384, frameSize);
                    DirectAudioDevice.DirectBAOS directBAOS = new DirectAudioDevice.DirectBAOS();
                    byte[] array;
                    try {
                        array = new byte[max];
                    }
                    catch (OutOfMemoryError outOfMemoryError) {
                        throw new IOException("Audio data is too big");
                    }
                    int i = 0;
                    while (i >= 0) {
                        i = audioInputStream.read(array, 0, array.length);
                        if (i > 0) {
                            directBAOS.write(array, 0, i);
                            off += i;
                        }
                        else {
                            if (i != 0) {
                                continue;
                            }
                            Thread.yield();
                        }
                    }
                    internalBuffer = directBAOS.getInternalBuffer();
                }
                this.open(audioInputStream.getFormat(), internalBuffer, off / frameSize);
            }
        }
        
        @Override
        public int getFrameLength() {
            return this.m_lengthInFrames;
        }
        
        @Override
        public long getMicrosecondLength() {
            return Toolkit.frames2micros(this.getFormat(), (long)this.getFrameLength());
        }
        
        @Override
        public void setFramePosition(int frameLength) {
            if (frameLength < 0) {
                frameLength = 0;
            }
            else if (frameLength >= this.getFrameLength()) {
                frameLength = this.getFrameLength();
            }
            if (this.doIO) {
                this.newFramePosition = frameLength;
            }
            else {
                this.clipBytePosition = frameLength * this.frameSize;
                this.newFramePosition = -1;
            }
            this.bytePosition = (long)(frameLength * this.frameSize);
            this.flush();
            synchronized (this.lockNative) {
                DirectAudioDevice.access$2500(this.id, this.isSource, (long)(frameLength * this.frameSize));
            }
        }
        
        @Override
        public long getLongFramePosition() {
            return super.getLongFramePosition();
        }
        
        @Override
        public synchronized void setMicrosecondPosition(long n) {
            this.setFramePosition((int)Toolkit.micros2frames(this.getFormat(), n));
        }
        
        @Override
        public void setLoopPoints(int i, int j) {
            if (i < 0 || i >= this.getFrameLength()) {
                throw new IllegalArgumentException("illegal value for start: " + i);
            }
            if (j >= this.getFrameLength()) {
                throw new IllegalArgumentException("illegal value for end: " + j);
            }
            if (j == -1) {
                j = this.getFrameLength() - 1;
                if (j < 0) {
                    j = 0;
                }
            }
            if (j < i) {
                throw new IllegalArgumentException("End position " + j + "  preceeds start position " + i);
            }
            this.loopStartFrame = i;
            this.loopEndFrame = j;
            return;
        }
        
        @Override
        public void loop(int loopCount) {
            this.loopCount = loopCount;
            this.start();
        }
        
        @Override
        void implOpen(AudioFormat audioFormat, int n) throws LineUnavailableException {
            if (this.audioData == null) {
                throw new IllegalArgumentException("illegal call to open() in interface Clip");
            }
            super.implOpen(audioFormat, n);
            return;
        }
        
        @Override
        void implClose() {
            Thread thread = this.thread;
            this.thread = null;
            this.doIO = false;
            if (thread != null) {
                synchronized (this.lock) {
                    this.lock.notifyAll();
                }
                try {
                    thread.join(2000L);
                }
                catch (InterruptedException ex) {}
            }
            super.implClose();
            this.audioData = null;
            this.newFramePosition = -1;
            this.getEventDispatcher().autoClosingClipClosed(this);
        }
        
        @Override
        void implStart() {
            super.implStart();
        }
        
        @Override
        void implStop() {
            super.implStop();
            this.loopCount = 0;
        }
        
        @Override
        public void run() {
            Thread currentThread = Thread.currentThread();
            while (this.thread == currentThread) {
                synchronized (this.lock) {
                    if (!this.doIO) {
                        try {
                            this.lock.wait();
                        }
                        catch (InterruptedException ex) {}
                        finally {
                            if (this.thread != currentThread) {
                                break;
                            }
                        }
                    }
                }
                while (this.doIO) {
                    if (this.newFramePosition >= 0) {
                        this.clipBytePosition = this.newFramePosition * this.frameSize;
                        this.newFramePosition = -1;
                    }
                    int loopEndFrame = this.getFrameLength() - 1;
                    if (this.loopCount > 0 || this.loopCount == -1) {
                        loopEndFrame = this.loopEndFrame;
                    }
                    int align = (int)((long)loopEndFrame - (long)(this.clipBytePosition / this.frameSize) + 1L) * this.frameSize;
                    if (align > this.getBufferSize()) {
                        align = Toolkit.align(this.getBufferSize(), this.frameSize);
                    }
                    int write = this.write(this.audioData, this.clipBytePosition, align);
                    this.clipBytePosition += write;
                    if (this.doIO && this.newFramePosition < 0 && write >= 0 && (long)(this.clipBytePosition / this.frameSize) > (long)loopEndFrame) {
                        if (this.loopCount > 0 || this.loopCount == -1) {
                            if (this.loopCount != -1) {
                                --this.loopCount;
                            }
                            this.newFramePosition = this.loopStartFrame;
                        }
                        else {
                            this.drain();
                            this.stop();
                        }
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        
        @Override
        public boolean isAutoClosing() {
            return this.autoclosing;
        }
        
        @Override
        public void setAutoClosing(boolean autoclosing) {
            if (autoclosing != this.autoclosing) {
                if (this.isOpen()) {
                    if (autoclosing) {
                        this.getEventDispatcher().autoClosingClipOpened(this);
                    }
                    else {
                        this.getEventDispatcher().autoClosingClipClosed(this);
                    }
                }
                this.autoclosing = autoclosing;
            }
        }
        
        @Override
        protected boolean requiresServicing() {
            return false;
        }
    }
    
    private static class DirectBAOS extends ByteArrayOutputStream
    {
        DirectBAOS() {
        }
        
        public byte[] getInternalBuffer() {
            return this.buf;
        }
    }
}
